"use strict";

exports.__esModule = true;
exports.buildComponentFromVNode = buildComponentFromVNode;
exports.renderComponent = renderComponent;
exports.setComponentProps = setComponentProps;
exports.unmountComponent = unmountComponent;
var _constants = require("../constants");
var _options = _interopRequireDefault(require("../options"));
var _util = require("../util");
var _renderQueue = require("../render-queue");
var _index = require("./index");
var _diff = require("./diff");
var _componentRecycler = require("./component-recycler");
var _index2 = require("../dom/index");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
/** Set a component's `props` (generally derived from JSX attributes).
 *	@param {Object} props
 *	@param {Object} [opts]
 *	@param {boolean} [opts.renderSync=false]	If `true` and {@link options.syncComponentUpdates} is `true`, triggers synchronous rendering.
 *	@param {boolean} [opts.render=true]			If `false`, no render will be triggered.
 */
function setComponentProps(component, props, opts, context, mountAll) {
  if (component._disable) return;
  component._disable = true;
  if (component.__ref = props.ref) delete props.ref;
  if (component.__key = props.key) delete props.key;
  if (!component.base || mountAll) {
    if (component.componentWillMount) component.componentWillMount();
  } else if (component.componentWillReceiveProps) {
    component.componentWillReceiveProps(props, context);
  }
  if (context && context !== component.context) {
    if (!component.prevContext) component.prevContext = component.context;
    component.context = context;
  }
  if (!component.prevProps) component.prevProps = component.props;
  component.props = props;
  component._disable = false;
  if (opts !== _constants.NO_RENDER) {
    if (opts === _constants.SYNC_RENDER || _options.default.syncComponentUpdates !== false || !component.base) {
      renderComponent(component, _constants.SYNC_RENDER, mountAll);
    } else {
      (0, _renderQueue.enqueueRender)(component);
    }
  }
  if (component.__ref) component.__ref(component);
}

/** Render a Component, triggering necessary lifecycle events and taking High-Order Components into account.
 *	@param {Component} component
 *	@param {Object} [opts]
 *	@param {boolean} [opts.build=false]		If `true`, component will build and store a DOM node if not already associated with one.
 *	@private
 */
function renderComponent(component, opts, mountAll, isChild) {
  if (component._disable) return;
  var props = component.props,
    state = component.state,
    context = component.context,
    previousProps = component.prevProps || props,
    previousState = component.prevState || state,
    previousContext = component.prevContext || context,
    isUpdate = component.base,
    nextBase = component.nextBase,
    initialBase = isUpdate || nextBase,
    initialChildComponent = component._component,
    skip = false,
    rendered,
    inst,
    cbase;

  // if updating
  if (isUpdate) {
    component.props = previousProps;
    component.state = previousState;
    component.context = previousContext;
    if (opts !== _constants.FORCE_RENDER && component.shouldComponentUpdate && component.shouldComponentUpdate(props, state, context) === false) {
      skip = true;
    } else if (component.componentWillUpdate) {
      component.componentWillUpdate(props, state, context);
    }
    component.props = props;
    component.state = state;
    component.context = context;
  }
  component.prevProps = component.prevState = component.prevContext = component.nextBase = null;
  component._dirty = false;
  if (!skip) {
    rendered = component.render(props, state, context);

    // context to pass to the child, can be updated via (grand-)parent component
    if (component.getChildContext) {
      context = (0, _util.extend)((0, _util.extend)({}, context), component.getChildContext());
    }
    var childComponent = rendered && rendered.nodeName,
      toUnmount,
      base;
    if (typeof childComponent === 'function') {
      // set up high order component link

      var childProps = (0, _index.getNodeProps)(rendered);
      inst = initialChildComponent;
      if (inst && inst.constructor === childComponent && childProps.key == inst.__key) {
        setComponentProps(inst, childProps, _constants.SYNC_RENDER, context, false);
      } else {
        toUnmount = inst;
        component._component = inst = (0, _componentRecycler.createComponent)(childComponent, childProps, context);
        inst.nextBase = inst.nextBase || nextBase;
        inst._parentComponent = component;
        setComponentProps(inst, childProps, _constants.NO_RENDER, context, false);
        renderComponent(inst, _constants.SYNC_RENDER, mountAll, true);
      }
      base = inst.base;
    } else {
      cbase = initialBase;

      // destroy high order component link
      toUnmount = initialChildComponent;
      if (toUnmount) {
        cbase = component._component = null;
      }
      if (initialBase || opts === _constants.SYNC_RENDER) {
        if (cbase) cbase._component = null;
        base = (0, _diff.diff)(cbase, rendered, context, mountAll || !isUpdate, initialBase && initialBase.parentNode, true);
      }
    }
    if (initialBase && base !== initialBase && inst !== initialChildComponent) {
      var baseParent = initialBase.parentNode;
      if (baseParent && base !== baseParent) {
        baseParent.replaceChild(base, initialBase);
        if (!toUnmount) {
          initialBase._component = null;
          (0, _diff.recollectNodeTree)(initialBase, false);
        }
      }
    }
    if (toUnmount) {
      unmountComponent(toUnmount);
    }
    component.base = base;
    if (base && !isChild) {
      var componentRef = component,
        t = component;
      while (t = t._parentComponent) {
        (componentRef = t).base = base;
      }
      base._component = componentRef;
      base._componentConstructor = componentRef.constructor;
    }
  }
  if (!isUpdate || mountAll) {
    _diff.mounts.unshift(component);
  } else if (!skip) {
    // Ensure that pending componentDidMount() hooks of child components
    // are called before the componentDidUpdate() hook in the parent.
    // Note: disabled as it causes duplicate hooks, see https://github.com/developit/preact/issues/750
    // flushMounts();

    if (component.componentDidUpdate) {
      component.componentDidUpdate(previousProps, previousState, previousContext);
    }
    if (_options.default.afterUpdate) _options.default.afterUpdate(component);
  }
  if (component._renderCallbacks != null) {
    while (component._renderCallbacks.length) component._renderCallbacks.pop().call(component);
  }
  if (!_diff.diffLevel && !isChild) (0, _diff.flushMounts)();
}

/** Apply the Component referenced by a VNode to the DOM.
 *	@param {Element} dom	The DOM node to mutate
 *	@param {VNode} vnode	A Component-referencing VNode
 *	@returns {Element} dom	The created/mutated element
 *	@private
 */
function buildComponentFromVNode(dom, vnode, context, mountAll) {
  var c = dom && dom._component,
    originalComponent = c,
    oldDom = dom,
    isDirectOwner = c && dom._componentConstructor === vnode.nodeName,
    isOwner = isDirectOwner,
    props = (0, _index.getNodeProps)(vnode);
  while (c && !isOwner && (c = c._parentComponent)) {
    isOwner = c.constructor === vnode.nodeName;
  }
  if (c && isOwner && (!mountAll || c._component)) {
    setComponentProps(c, props, _constants.ASYNC_RENDER, context, mountAll);
    dom = c.base;
  } else {
    if (originalComponent && !isDirectOwner) {
      unmountComponent(originalComponent);
      dom = oldDom = null;
    }
    c = (0, _componentRecycler.createComponent)(vnode.nodeName, props, context);
    if (dom && !c.nextBase) {
      c.nextBase = dom;
      // passing dom/oldDom as nextBase will recycle it if unused, so bypass recycling on L229:
      oldDom = null;
    }
    setComponentProps(c, props, _constants.SYNC_RENDER, context, mountAll);
    dom = c.base;
    if (oldDom && dom !== oldDom) {
      oldDom._component = null;
      (0, _diff.recollectNodeTree)(oldDom, false);
    }
  }
  return dom;
}

/** Remove a component from the DOM and recycle it.
 *	@param {Component} component	The Component instance to unmount
 *	@private
 */
function unmountComponent(component) {
  if (_options.default.beforeUnmount) _options.default.beforeUnmount(component);
  var base = component.base;
  component._disable = true;
  if (component.componentWillUnmount) component.componentWillUnmount();
  component.base = null;

  // recursively tear down & recollect high-order component children:
  var inner = component._component;
  if (inner) {
    unmountComponent(inner);
  } else if (base) {
    if (base[_constants.ATTR_KEY] && base[_constants.ATTR_KEY].ref) base[_constants.ATTR_KEY].ref(null);
    component.nextBase = base;
    (0, _index2.removeNode)(base);
    (0, _componentRecycler.collectComponent)(component);
    (0, _diff.removeChildren)(base);
  }
  if (component.__ref) component.__ref(null);
}